[pacman-dev] [PATCH] Add new SharedPkgCache option.

Chantry Xavier shiningxc at gmail.com
Sat Feb 16 10:06:10 EST 2008


As it was already mentioned several times, the new -Sc behavior in 3.1 is
great, but only when the package cache is not shared.

When this option is enabled, -Sc will clean packages that are no longer
available in any sync db, rather than packages that are no longer in the
local db. The resulting behavior should be better for shared cache.

Ref :
http://www.archlinux.org/pipermail/pacman-dev/2008-February/011140.html

Signed-off-by: Chantry Xavier <shiningxc at gmail.com>
---
 doc/pacman.8.txt      |    3 ++
 doc/pacman.conf.5.txt |    8 ++++++
 src/pacman/conf.h     |    9 +++++--
 src/pacman/pacman.c   |    3 ++
 src/pacman/sync.c     |   58 +++++++++++++++++++++++++++++++++---------------
 5 files changed, 60 insertions(+), 21 deletions(-)

diff --git a/doc/pacman.8.txt b/doc/pacman.8.txt
index f6eb69c..611fa3e 100644
--- a/doc/pacman.8.txt
+++ b/doc/pacman.8.txt
@@ -241,6 +241,9 @@ Sync Options[[SO]]
 	packages that are no longer installed; use two to remove all packages
 	from the cache. In both cases, you will have a yes or no option to
 	remove packages and/or unused downloaded databases.
++
+If you use a network shared cache, see the 'SharedPkgCache' option in
+linkman:pacman.conf[5].
 
 *-e, \--dependsonly*::
 	Install all dependencies of a package, but not the specified package
diff --git a/doc/pacman.conf.5.txt b/doc/pacman.conf.5.txt
index e8f7454..c1b76e8 100644
--- a/doc/pacman.conf.5.txt
+++ b/doc/pacman.conf.5.txt
@@ -131,6 +131,14 @@ Options
 	than the percent of each individual download target. The progress
 	bar is still based solely on the current file download.
 
+*SharedPkgCache*::
+	If set, the '-Sc' operation will clean outdated packages (not present in
+	any sync database), rather than packages that are no longer installed
+	(not present in the local database).
+	This is especially useful when the package cache is shared among multiple
+	machines, where the local databases are usually different, but the sync
+	databases used could be the same.
+
 Repository Sections
 -------------------
 Each repository section defines a section name and at least one location where
diff --git a/src/pacman/conf.h b/src/pacman/conf.h
index f804f56..a54a9f1 100644
--- a/src/pacman/conf.h
+++ b/src/pacman/conf.h
@@ -67,9 +67,12 @@ typedef struct __config_t {
 	unsigned short chomp; /* I Love Candy! */
 	unsigned short usecolor; /* enable colorful output */
 	unsigned short showsize; /* show individual package sizes */
-	unsigned short totaldownload; /* When downloading, display the amount
-	                                 downloaded, rate, ETA, and percent
-	                                 downloaded of the total download list */
+	/* When downloading, display the amount downloaded, rate, ETA, and percent
+	 * downloaded of the total download list */
+	unsigned short totaldownload;
+	/* if the cache is shared, -Sc will clean outdated packages rather than
+	 * packages that are no longer installed */
+	unsigned short sharedpkgcache;
 } config_t;
 
 /* Operations */
diff --git a/src/pacman/pacman.c b/src/pacman/pacman.c
index 377ea3f..7753833 100644
--- a/src/pacman/pacman.c
+++ b/src/pacman/pacman.c
@@ -638,6 +638,9 @@ static int _parseconfig(const char *file, const char *givensection,
 				} else if(strcmp(key, "TotalDownload") == 0 || strcmp(upperkey, "TOTALDOWNLOAD") == 0) {
 					config->totaldownload = 1;
 					pm_printf(PM_LOG_DEBUG, "config: totaldownload\n");
+				} else if(strcmp(key, "SharedPkgCache") == 0 || strcmp(upperkey, "SHAREDPKGCACHE") == 0) {
+					config->sharedpkgcache = 1;
+					pm_printf(PM_LOG_DEBUG, "config: usedelta\n");
 				} else {
 					pm_printf(PM_LOG_ERROR, _("config file %s, line %d: directive '%s' not recognized.\n"),
 							file, linenum, key);
diff --git a/src/pacman/sync.c b/src/pacman/sync.c
index 27218d6..cf11710 100644
--- a/src/pacman/sync.c
+++ b/src/pacman/sync.c
@@ -125,13 +125,17 @@ static int sync_cleancache(int level)
 		/* incomplete cleanup */
 		DIR *dir;
 		struct dirent *ent;
-		/* Let's vastly improve the way this is done. Before, we went by package
-		 * name. Instead, let's only keep packages we have installed. Open up each
-		 * package and see if it has an entry in the local DB; if not, delete it.
-		 */
+		/* Open up each package and see if it has an entry in the local DB; if not,
+		 * delete it. */
 		printf(_("Cache directory: %s\n"), cachedir);
-		if(!yesno(_("Do you want to remove uninstalled packages from cache? [Y/n] "))) {
-			return(0);
+		if(config->sharedpkgcache) {
+			if(!yesno(_("Do you want to remove outdated packages from cache? [Y/n] "))) {
+				return(0);
+			}
+		} else {
+			if(!yesno(_("Do you want to remove uninstalled packages from cache? [Y/n] "))) {
+				return(0);
+			}
 		}
 		printf(_("removing old packages from cache... "));
 
@@ -145,13 +149,14 @@ static int sync_cleancache(int level)
 		/* step through the directory one file at a time */
 		while((ent = readdir(dir)) != NULL) {
 			char path[PATH_MAX];
-			pmpkg_t *localpkg = NULL, *dbpkg = NULL;
+			int delete = 1;
+			pmpkg_t *localpkg = NULL, *pkg = NULL;
 
 			if(!strcmp(ent->d_name, ".") || !strcmp(ent->d_name, "..")) {
 				continue;
 			}
 			/* build the full filepath */
-			snprintf(path, PATH_MAX, "%s/%s", cachedir, ent->d_name);
+			snprintf(path, PATH_MAX, "%s%s", cachedir, ent->d_name);
 
 			/* attempt to load the package, skip file on failures as we may have
 			 * files here that aren't valid packages. we also don't need a full
@@ -159,19 +164,36 @@ static int sync_cleancache(int level)
 			if(alpm_pkg_load(path, 0, &localpkg) != 0 || localpkg == NULL) {
 				continue;
 			}
-			/* check if this package is in the local DB */
-			dbpkg = alpm_db_get_pkg(db_local, alpm_pkg_get_name(localpkg));
-			if(dbpkg == NULL) {
-				/* delete package, not present in local DB */
-				unlink(path);
-			} else if(alpm_pkg_vercmp(alpm_pkg_get_version(localpkg),
-							alpm_pkg_get_version(dbpkg)) != 0) {
-				/* delete package, it was found but version differs */
-				unlink(path);
+			if(config->sharedpkgcache) {
+				/* check if this package is in a sync DB */
+				alpm_list_t *sync_dbs = alpm_option_get_syncdbs();
+				alpm_list_t *j;
+
+				for(j = sync_dbs; j; j = alpm_list_next(j)) {
+					pmdb_t *db = alpm_list_getdata(j);
+					pkg = alpm_db_get_pkg(db, alpm_pkg_get_name(localpkg));
+					if(pkg != NULL && alpm_pkg_vercmp(alpm_pkg_get_version(localpkg),
+								alpm_pkg_get_version(pkg)) == 0) {
+						/* package was found in a sync DB and version matches, keep it */
+						delete = 0;
+						break;
+					}
+				}
+			} else {
+				/* check if this package is in the local DB */
+				pkg = alpm_db_get_pkg(db_local, alpm_pkg_get_name(localpkg));
+				if(pkg != NULL && alpm_pkg_vercmp(alpm_pkg_get_version(localpkg),
+							alpm_pkg_get_version(pkg)) == 0) {
+					/* package was found in local DB and version matches, keep it */
+					delete = 0;
+				}
 			}
-			/* else version was the same, so keep the package */
 			/* free the local file package */
 			alpm_pkg_free(localpkg);
+
+			if(delete) {
+				unlink(path);
+			}
 		}
 		printf(_("done.\n"));
 	} else {
-- 
1.5.4





More information about the pacman-dev mailing list